searchData={"items":[{"type":"module","title":"TaggedTupleShorthand","doc":"> **_Field punning in Elixir via a shorthand for constructing tagged two-tuple variable references._**\n\n\n\n> #### `use TaggedTupleShorthand` {: .info}\n>\n> When you `use TaggedTupleShorthand`, you are replacing `Kernel.@/1` with:\n> - an overloaded `TaggedTupleShorthand.@/1` implementation\n> - that supports `@:atom` and `@\"string\"` tagged tuple variable references\n> - and otherwise falls back to normal `@module_attribute` semantics","ref":"TaggedTupleShorthand.html"},{"type":"module","title":"Field Punning - TaggedTupleShorthand","doc":"<!--\n  all hyperlinks within this snippet must be inline,\n  rather than using markdown link references\n-->\n\nWhat is field punning? It's a common form of syntactic sugar you may already be familiar with from other languages. It goes by many names:\n\n- [Field Punning](https://dev.realworldocaml.org/records.html) — OCaml\n- [Record Puns](https://ghc.gitlab.haskell.org/ghc/doc/users_guide/exts/record_puns.html) — Haskell\n- [Object Property Value Shorthand](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Object_initializer#property_definitions) — ES6 Javascript\n- [Hash Key Pattern Matching](https://docs.ruby-lang.org/en/3.0/syntax/pattern_matching_rdoc.html#label-Matching+non-primitive+objects-3A+deconstruct+and+deconstruct_keys) — Ruby\n\nWe'll stick with \"field punning\" throughout this explanation.","ref":"TaggedTupleShorthand.html#module-field-punning"},{"type":"module","title":"Background - TaggedTupleShorthand","doc":"We often use `Keyword` lists and `Map`s to associate values with a given key:\n\n```elixir\nlist = [foo: 1, bar: 2]\nmap = %{fizz: 3, buzz: 4}\n```\n\nOften, we want to get values of interest associated with a given key out of an associative data structure. There are functions as well as syntax sugar for this already:\n\n```elixir\nKeyword.get(list, :foo) #=> 1\nlist[:bar] #=> 2\nmap[:fizz] #=> 3\nmap.buzz #=> 4\n```\n\nIf we're interested in a value, we are probably going to assign it to a variable. What's a good name for that variable? 94% of the time[‡](https://en.wikipedia.org/wiki/Citation_needed), the key itself makes for a fine variable name:\n\n```elixir\nfoo = Keyword.get(list, :foo)\nbar = list[:bar]\nfizz = map[:fizz]\nbuzz = map.buzz\n```\n\nAnd thanks to the glory of pattern matching, we can express this with destructuring:\n\n```elixir\n[foo: foo, bar: bar] = list\n%{fizz: fizz, buzz: buzz} = map\nfoo #=> 1\nbar #=> 2\nfizz #=> 3\nbuzz #=> 4\n```\n\nThis begs the question: if this is so common, ***why do we have to type out the same name twice***, *once to name the key, and again to name the variable*, when destructuring?\n\n#### In Javascript\n\nYou can do this destructuring of key/value pairs into matching variable names by assigning to a \"barewords\" style object literal:\n\n```js\ndata = {foo: 1, bar: 2, baz: 3}\n//=> {foo: 1, bar: 2, baz: 3}\n{foo, bar} = data\nfoo //=> 1\nbar //=> 2\n```\n\n#### In Ruby\n\nYou can do this destructuring of key/value pairs into matching variable names by pattern matching into a \"keywords\" style hash literal:\n\n```rb\ndata = {foo: 1, bar: 2, baz: 3}\n#=> {:foo=>1, :bar=>2, :baz=>3}\ndata => {foo:, bar:}\nfoo #=> 1\nbar #=> 2\n```\n\n#### Benefits\n\nThat is what *field punning* is: ***a short-hand syntactic sugar for deconstruction of key/value pairs in associative data structures, interacting with variable names in the current scope***. It is popular for several reasons:\n\n- This syntax saves on visual noise, expressing destructuring key/value data tersely in the common case of the key making for a sufficient variable name.\n- This syntax calls attention to the cases where we are intentionally *not* re-using the key as a variable name, placing emphasis on a subtle decision a developer decided was important for readability or understanding.\n- This syntax prevents common typos, and ensures that variable names match keys throughout refactors when that is the desired behaviour.\n\n#### In Elixir\n\nAn Elixir implementation of field punning has to work in several more scenarios than other languages, since:\n\n- We have two different common associative data structures, `Keyword` lists and `Map`s\n- We have two different common key types, `Atom`s and `String`s\n- We have two different common syntaxes for key/value associativity, `arbitrary => value` (maps only) and `atom: value` (atom keys only)\n\nThis particular macro for tagged two-tuple variable references gets us just that.","ref":"TaggedTupleShorthand.html#module-background"},{"type":"module","title":"Usage - TaggedTupleShorthand","doc":"<!--\n  all hyperlinks within this snippet must be inline,\n  rather than using markdown link references\n-->","ref":"TaggedTupleShorthand.html#module-usage"},{"type":"module","title":"Basic Usage - TaggedTupleShorthand","doc":"`TaggedTupleShorthand` overrides the `@` operator to accept a literal atom or string, that turns into a tagged two-tuple variable reference at compile-time:\n\nForm              | Expands To\n------------------|-----------\n`@:atom`          | `{:atom, atom}`\n`@^:atom`         | `{:atom, ^atom}`\n`@\"string\"`       | `{\"string\", string}`\n`@^\"string\"`      | `{\"string\", ^string}`\n`@anything_else`  | Fallback to `Kernel.@/1`\n\n#### Examples\n\n    iex> use TaggedTupleShorthand\n    iex> foo = 1\n    iex> @:foo\n    {:foo, 1}\n    iex> @:foo = {:foo, 2}\n    {:foo, 2}\n    iex> foo\n    2\n    iex> @^:foo = {:foo, 2}\n    iex> @^:foo = {:foo, 3}\n    ** (MatchError) no match of right hand side value: {:foo, 3}\n\n      \nThis is not the most useful construct, until we start to use it in destructuring.","ref":"TaggedTupleShorthand.html#module-basic-usage"},{"type":"module","title":"Field Punning Usage - TaggedTupleShorthand","doc":"As it so happens, this tagged two-tuple variable reference shorthand expands at compile-time to AST that gives us field punning. Just use `@:atom` and `@\"string\"` when destructuring:\n\n    iex> use TaggedTupleShorthand\n    iex> destructure_map = fn %{@:foo, @\"bar\"} ->\n    ...>   {foo, bar}\n    ...> end\n    iex> map = %{\"bar\" => 2, foo: 1}\n    iex> destructure_map.(map)\n    {1, 2}\n\nSome more realistic examples:\n\n#### In Phoenix Channels\n\n[Before](https://groups.google.com/g/elixir-lang-core/c/NoUo2gqQR3I/m/ddgTD3DU4oMJ):\n\n```elixir\ndef handle_in(\n      event,\n      %{\n        \"chat\" => chat,\n        \"question_id\" => question_id,\n        \"data\" => data,\n        \"attachment\" => attachment\n      },\n      socket\n    )\n    when is_binary(chat) do...\n```\n\nAfter:\n\n```elixir\ndef handle_in(event, %{@\"chat\", @\"question_id\", @\"data\", @\"attachment\"}, socket)\n    when is_binary(chat) do...\n```\n\nDiff:\n\n```diff\n-def handle_in(\n-      event,\n-      %{\n-        \"chat\" => chat,\n-        \"question_id\" => question_id,\n-        \"data\" => data,\n-        \"attachment\" => attachment\n-      },\n-      socket\n-    )\n+def handle_in(event, %{@\"chat\", @\"question_id\", @\"data\", @\"attachment\"}, socket)\n     when is_binary(chat) do...\n```\n\n#### In Phoenix Controller Actions\n\n[Before](https://github.com/fly-apps/live_beats/blob/ac9780472e7019af274110a1cf71250a8d40c986/lib/live_beats_web/controllers/file_controller.ex#L11-L20):\n\n```elixir\ndef show(conn, %{\"id\" => id, \"token\" => token}) do\n  case Phoenix.Token.decrypt(conn, \"file\", token, max_age: :timer.minutes(1)) do\n    {:ok, %{id: ^id, vsn: 1, size: _size}} ->\n     path = MediaLibrary.local_filepath(id)\n     do_send_file(conn, path)\n\n    _ ->\n      send_resp(conn, :unauthorized, \"\")\n  end\nend\n```\n\nAfter:\n\n```elixir\ndef show(conn, %{@\"id\", @\"token\"}) do\n  case Phoenix.Token.decrypt(conn, \"file\", token, max_age: :timer.minutes(1)) do\n    {:ok, %{@^:id, vsn: 1, size: _size}} ->\n     path = MediaLibrary.local_filepath(id)\n     do_send_file(conn, path)\n\n    _ ->\n      send_resp(conn, :unauthorized, \"\")\n  end\nend\n```\n\nDiff:\n\n```diff\n-def show(conn, %{\"id\" => id, \"token\" => token}) do\n+def show(conn, %{@\"id\", @\"token\"}) do\n   case Phoenix.Token.decrypt(conn, \"file\", token, max_age: :timer.minutes(1)) do\n-    {:ok, %{id: ^id, vsn: 1, size: _size}} ->\n+    {:ok, %{@^:id, vsn: 1, size: _size}} ->\n      path = MediaLibrary.local_filepath(id)\n      do_send_file(conn, path)\n```","ref":"TaggedTupleShorthand.html#module-field-punning-usage"},{"type":"module","title":"Extras - TaggedTupleShorthand","doc":"<!--\n  all hyperlinks within this snippet must be inline,\n  rather than using markdown link references\n-->","ref":"TaggedTupleShorthand.html#module-extras"},{"type":"module","title":"Formatting - TaggedTupleShorthand","doc":"At time of writing, this library does not do any custom formatting, but that will likely change. To get support for it on release, you can add `:tagged_tuple_shorthand` to your formatter options' `:import_deps` today, ex:\n\n```elixir\n# project/.formatter.exs\n[\n  import_deps: [:tagged_tuple_shorthand]\n]\n```","ref":"TaggedTupleShorthand.html#module-formatting"},{"type":"module","title":"Linting - TaggedTupleShorthand","doc":"At time of writing, `Credo` is reasonably upset by how we re-appropriate the module attribute operator. We may offer a replacement check in the future, but for now you should disable the `Credo.Check.Readability.ModuleAttributeNames` check in your configuration, ex:\n\n```elixir\n# project/.credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      checks: %{\n        disabled: [\n          {Credo.Check.Readability.ModuleAttributeNames, false}\n        ]\n      }\n    }\n  ]\n}\n```","ref":"TaggedTupleShorthand.html#module-linting"},{"type":"macro","title":"TaggedTupleShorthand.@/1","doc":"Generates tagged two-tuple variable references from atom and string literals.\n\nOtherwise falls back to `Kernel.@/1`:\n\nForm              | Expands To\n------------------|-----------\n`@:atom`          | `{:atom, atom}`\n`@^:atom`         | `{:atom, ^atom}`\n`@\"string\"`       | `{\"string\", string}`\n`@^\"string\"`      | `{\"string\", ^string}`\n`@anything_else`  | Fallback to `Kernel.@/1`","ref":"TaggedTupleShorthand.html#@/1"},{"type":"macro","title":"Examples - TaggedTupleShorthand.@/1","doc":"iex> use TaggedTupleShorthand\n    iex> foo = 1\n    iex> @:foo\n    {:foo, 1}\n    iex> @:foo = {:foo, 2}\n    {:foo, 2}\n    iex> foo\n    2\n    iex> @^:foo = {:foo, 2}\n    iex> @^:foo = {:foo, 3}\n    ** (MatchError) no match of right hand side value: {:foo, 3}\n\nIntended to be used in pattern matching constructs to enable field punning,\nsee the module documentation for an explanation of\n[field punning](https://hexdocs.pm/tagged_tuple_shorthand/TaggedTupleShorthand.html#module-field-punning)\nand its\n[intended usage](https://hexdocs.pm/tagged_tuple_shorthand/TaggedTupleShorthand.html#module-field-punning-usage).","ref":"TaggedTupleShorthand.html#@/1-examples"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}